@@ -96,7 +96,9 @@ class Request(BaseRequest):
     """
     __slots__: tuple[str, ...] = ("_method", "_params", "_id")
 
-    def __init__(self, *,
+    def __init__(
+        self,
+        *,
         method: str,
         params: _Params | UndefinedType = Undefined,
         request_id: str | float | UndefinedType = Undefined
