@@ -87,18 +87,24 @@ class Response(BaseResponse):
     __slots__: tuple[str, ...] = ("_body", "_error", "_id")
 
     @overload
-    def __init__(self, *,
+    def __init__(
+        self,
+        *,
         body: Any,
         response_id: str | float | UndefinedType | None = ...
     ) -> None: ...
 
     @overload
-    def __init__(self, *,
+    def __init__(
+        self,
+        *,
         error: Error,
         response_id: str | float | UndefinedType | None = ...
     ) -> None: ...
 
-    def __init__(self, *,
+    def __init__(
+        self,
+        *,
         body: Any = Undefined,
         error: Error | UndefinedType = Undefined,
         response_id: str | float | UndefinedType | None = Undefined
