@@ -14,9 +14,8 @@
 # limitations under the License.
 
 from abc import ABCMeta, abstractmethod
-from io import BytesIO, StringIO
-from json import dump as _json_encode, load as _json_decode
-from pickle import HIGHEST_PROTOCOL, dump as _pickle_encode, load as _pickle_decode
+from json import dumps as _json_encode, loads as _json_decode
+from pickle import HIGHEST_PROTOCOL, dumps as _pickle_encode, loads as _pickle_decode
 from typing import Any, ClassVar, Final
 
 from ._errors import Error, ErrorEnum
@@ -53,12 +52,10 @@ class JSONSerializer(BaseSerializer):
         :raises jsonrpc.Error: If any exception has occurred due the serialization or/and encoding to :py:class:`bytes`.
         :returns: The :py:class:`bytes` object containing the serialized Python data structure.
         """
-        with StringIO() as raw_buffer:
-            try:
-                _json_encode(obj, raw_buffer, separators=(",", ":"))
-                return raw_buffer.getvalue().encode("ascii")
-            except Exception as exc:
-                raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to serialize object to JSON") from exc
+        try:
+            return _json_encode(obj, separators=(",", ":")).encode("latin-1")
+        except Exception as exc:
+            raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to serialize object to JSON") from exc
 
     def deserialize(self, obj: bytes) -> Any:
         """
@@ -68,11 +65,10 @@ class JSONSerializer(BaseSerializer):
         :raises jsonrpc.Error: If any exception has occurred due the deserialization or/and decoding from :py:class:`bytes`.
         :returns: An any type of object containing the deserialized Python data structure.
         """
-        with BytesIO(obj) as raw_buffer:
-            try:
-                return _json_decode(raw_buffer)
-            except Exception as exc:
-                raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to deserialize object from JSON") from exc
+        try:
+            return _json_decode(obj)
+        except Exception as exc:
+            raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to deserialize object from JSON") from exc
 
 
 class PickleSerializer(BaseSerializer):
@@ -93,12 +89,10 @@ class PickleSerializer(BaseSerializer):
         :raises jsonrpc.Error: If exception has occurred due the pickling.
         :returns: The :py:class:`bytes` object containing the pickled object.
         """
-        with BytesIO() as raw_buffer:
-            try:
-                _pickle_encode(obj, raw_buffer, self.PROTOCOL_VERSION)
-                return raw_buffer.getvalue()
-            except Exception as exc:
-                raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to serialize object") from exc
+        try:
+            return _pickle_encode(obj, self.PROTOCOL_VERSION)
+        except Exception as exc:
+            raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to serialize object") from exc
 
     def deserialize(self, obj: bytes) -> Any:
         """
@@ -108,8 +102,7 @@ class PickleSerializer(BaseSerializer):
         :raises jsonrpc.Error: If exception has occurred due the unpickling.
         :returns: An any type of object containing the deserialized Python data structure.
         """
-        with BytesIO(obj) as raw_buffer:
-            try:
-                return _pickle_decode(raw_buffer)
-            except Exception as exc:
-                raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to deserialize object") from exc
+        try:
+            return _pickle_decode(obj)
+        except Exception as exc:
+            raise Error(code=ErrorEnum.PARSE_ERROR, message="Failed to deserialize object") from exc
