@@ -16,13 +16,13 @@
 from abc import ABCMeta, abstractmethod
 from collections import UserDict as Dict
 from collections.abc import Iterator, MutableSequence
-from functools import partial, singledispatch
+from concurrent.futures import Executor, Future, ThreadPoolExecutor
+from functools import partial
 from http import HTTPStatus
-from io import DEFAULT_BUFFER_SIZE, BytesIO
+from io import DEFAULT_BUFFER_SIZE
 from sys import exc_info
-from threading import Thread
 from traceback import print_exception
-from typing import Any, ClassVar, Final, Protocol, TypeAlias, cast, overload
+from typing import Any, ClassVar, Final, TypeAlias
 
 from ._dispatcher import Dispatcher
 from ._errors import BaseError, Error
@@ -33,17 +33,8 @@ from ._typing import Headers, InputStream, OptExcInfo, StartResponse, WSGIEnviro
 
 __all__: Final[tuple[str, ...]] = ("WSGIHandler",)
 
-_AnyRequest: TypeAlias = BaseRequest | BaseError | BaseBatchRequest
-_AnyResponse: TypeAlias = BaseResponse | BaseBatchResponse | None
-
-
-class request_handler(Protocol):
-    @overload
-    def __call__(self, request: BaseRequest) -> BaseResponse | None: ...
-    @overload
-    def __call__(self, request: BaseError) -> BaseResponse: ...
-    @overload
-    def __call__(self, request: BaseBatchRequest[Any]) -> BaseBatchResponse[Any]: ...
+_AnyRequest: TypeAlias = BaseRequest | BaseError | BaseBatchRequest[Any]
+_AnyResponse: TypeAlias = BaseResponse | BaseBatchResponse[Any] | None
 
 
 class BaseWSGIHandler(Dict[str, Any], metaclass=ABCMeta):
@@ -67,7 +58,7 @@ class BaseWSGIHandler(Dict[str, Any], metaclass=ABCMeta):
             return wsgi_response(status=HTTPStatus.METHOD_NOT_ALLOWED)
 
         try:
-            if not (request_body := self._read_request_body(environ)):
+            if not (request_body := b"".join(self._read_request_body(environ))):
                 # Trying to check the request body is empty.
                 # If that's true then it returns HTTP 400 "Bad Request".
                 return wsgi_response(status=HTTPStatus.BAD_REQUEST)
@@ -86,28 +77,27 @@ class BaseWSGIHandler(Dict[str, Any], metaclass=ABCMeta):
             print_exception(exc, file=environ["wsgi.errors"])
             return wsgi_response(status=HTTPStatus.INTERNAL_SERVER_ERROR, exc_info=exc_info())
 
-    def _read_request_body(self, environ: WSGIEnvironment) -> bytes:
+    def _read_request_body(self, environ: WSGIEnvironment) -> Iterator[bytes]:
         try:
             content_length: int = int(environ["CONTENT_LENGTH"])
         except (KeyError, ValueError):
-            return b""
+            return
 
         stream: Final[InputStream] = environ["wsgi.input"]
 
-        with BytesIO() as raw_buffer:
-            # Ensure to disallow reading the stream more bytes
-            # than specified by "Content-Length" header:
-            while content_length > 0:
-                if not (chunk := stream.read(min(content_length, DEFAULT_BUFFER_SIZE))):
-                    raise EOFError(f"Client disconnected, {content_length:d} more bytes were expected")
+        # Ensure to disallow reading the stream more bytes
+        # than specified by "Content-Length" header:
+        while content_length > 0:
+            if not (chunk := stream.read(min(content_length, DEFAULT_BUFFER_SIZE))):
+                raise EOFError(f"Client disconnected, {content_length:d} more bytes were expected")
 
-                # Appends the chunk of request body to the buffer
-                # and decreases the request size:
-                content_length -= raw_buffer.write(chunk)
+            # Yields the chunk of request body to the iterator
+            # and decreases the request size:
+            yield chunk
+            content_length -= len(chunk)
 
-            return raw_buffer.getvalue()
-
-    def _get_response(self,
+    def _get_response(
+        self,
         start_response: StartResponse,
         *,
         status: HTTPStatus = HTTPStatus.OK,
@@ -127,9 +117,8 @@ class BaseWSGIHandler(Dict[str, Any], metaclass=ABCMeta):
         start_response(f"{status.value:d}\u0020{status.phrase!s}", headers, exc_info)
         yield response_body
 
-    @property
     @abstractmethod
-    def handle_request(self) -> request_handler:
+    def handle_request(self, deserialized_object: _AnyRequest) -> _AnyResponse:
         raise NotImplementedError
 
     @abstractmethod
@@ -160,50 +149,52 @@ class WSGIHandler(BaseWSGIHandler):
     #: used by this class for data serialization by default.
     serializer: ClassVar[JSONSerializer] = JSONSerializer()
 
-    @property
-    def handle_request(self) -> request_handler:
-        """
-        Returns a single-dispatch method for the handling deserialized requests.
+    @classmethod
+    def _handle_deserialized_request(cls, executor: Executor, request: Request) -> Response | None:
+        future: Final[Future[Any]] = executor.submit(cls.dispatcher.dispatch, request.method, *request.args, **request.kwargs)
+        if request.is_notification:
+            return None
+        try:
+            result: Final[Any] = future.result()
+            return Response(body=result, response_id=request.request_id)
+        except Error as error:
+            return Response(error=error, response_id=request.request_id)
 
-        :param request: One of the following objects types: :class:`jsonrpc.Request`, :class:`jsonrpc.BatchRequest`
+    @staticmethod
+    def _handle_deserialized_error(error: Error) -> Response:
+        return Response(error=error, response_id=None)
+
+    @classmethod
+    def _handle_deserialized_batch_request(cls, executor: Executor, batch_request: BatchRequest) -> BatchResponse:
+        def wrapper(obj: Request | Error) -> Response | None:
+            if isinstance(obj, Request):
+                return cls._handle_deserialized_request(executor, obj)
+            else:
+                return cls._handle_deserialized_error(obj)
+
+        return BatchResponse(filter(None, executor.map(wrapper, batch_request)))
+
+    def handle_request(self, deserialized_object: _AnyRequest) -> _AnyResponse:
+        """
+        Base method for handling deserialized requests.
+
+        :param deserialized_object: One of the following objects types: :class:`jsonrpc.Request`, :class:`jsonrpc.BatchRequest`
             or :class:`jsonrpc.Error`. If the :class:`jsonrpc.BatchRequest` object type was supplied,
             its will be invoked itself recursively for each one item containing in the batch request.
         :raises ValueError: Due the invocation with unsupported request type.
         :returns: Either :class:`jsonrpc.Response` or :class:`jsonrpc.BatchResponse`.
             If the :class:`jsonrpc.Request` object was received and it's a notification, then it returns :py:data:`None`.
         """
-        @singledispatch
-        def wrapper(request: _AnyRequest) -> _AnyResponse:
-            raise ValueError(f"Unsupported type {type(request).__name__!r}")
-
-        @wrapper.register
-        def _(request: Request) -> Response | None:
-            # Save the callback for future use:
-            dispatch: partial[Any] = partial(self.dispatcher.dispatch, request.method)
-            # Redirect the method invocation to the separate thread,
-            # if the method is notification:
-            if request.is_notification:
-                Thread(target=dispatch, args=request.args, kwargs=request.kwargs, daemon=True).start()
-                return None
-            # Elsewise, execute the method as claimed and returns the error if an exception is raised:
-            try:
-                result: Final[Any] = dispatch(*request.args, **request.kwargs)
-                return Response(body=result, response_id=request.request_id)
-            except Error as error:
-                return Response(error=error, response_id=request.request_id)
-
-        @wrapper.register
-        def _(request: Error) -> Response:
-            # Returns the response as is with the error attribute.
-            return Response(error=request, response_id=None)
-
-        @wrapper.register
-        def _(request: BatchRequest) -> BatchResponse:
-            # We need to go deeper, 'cause received a batch request object.
-            # Then invoke itself recursively.
-            return BatchResponse(response for response in map(wrapper, request) if isinstance(response, Response))
-
-        return cast(request_handler, wrapper)
+        with ThreadPoolExecutor() as executor:
+            match deserialized_object:
+                case Request() as request:
+                    return self._handle_deserialized_request(executor, request)
+                case Error() as error:
+                    return self._handle_deserialized_error(error)
+                case BatchRequest() as batch_request:
+                    return self._handle_deserialized_batch_request(executor, batch_request)
+                case _:
+                    raise ValueError(f"Unsupported type {type(deserialized_object).__name__!r}")
 
     def process_request(self, request_body: bytes) -> bytes:
         """
